import { DatabaseHelper } from "./DatabaseHelper";
import { USER_TABLE } from "./AccountHelper";
import { Friend } from "../entities/Friend";

/**
 * Friends helper
 * 
 * @author Pierre HUBERT
 */

const FRIENDS_TABLE = "amis";

export class FriendsHelper {

	/**
	 * Get the list of friends of the user
	 * 
	 * @param userID The ID of the user
	 */
	public static async GetList(userID: number) : Promise<Array<Friend>> {
		const results = await DatabaseHelper.Query({
			table: FRIENDS_TABLE + " f",
			where: {
				ID_personne: userID,
			},
			joins: [
				{
					table: USER_TABLE + " u",
					condition: "f.ID_amis = u.ID"
				}
			],
			fields: [
				"u.last_activity",
				"f.ID_amis",
				"f.actif",
				"f.abonnement",
				"f.autoriser_post_page"
			],
			order: "u.last_activity DESC"
		});

		return results.map((r) => this.DbToFriend(r));
	}

	/**
	 * Get information about a single membership
	 * 
	 * @param userID Target user ID (ID of the user making the request)
	 * @param friendID Target friend ID
	 * @returns Information about the membership / null if no membership was found
	 */
	public static async GetSingleInfo(userID: number, friendID: number) : Promise<Friend> {
		
		// Same request has above, but just for a single user
		const result = await DatabaseHelper.QueryRow({
			table: FRIENDS_TABLE + " f",
			where: {
				ID_personne: userID,
				ID_amis: friendID
			},
			joins: [
				{
					table: USER_TABLE + " u",
					condition: "f.ID_amis = u.ID"
				}
			],
			fields: [
				"u.last_activity",
				"f.ID_amis",
				"f.actif",
				"f.abonnement",
				"f.autoriser_post_page"
			],
			order: "u.last_activity DESC"
		});

		if(result == null)
			return null;

		return this.DbToFriend(result);
	}

	/**
	 * Count the number of friendship requests a user
	 * received
	 * 
	 * @param userID Target user ID
	 */
	public static async CountRequests(userID: number) : Promise<number> {
		return await DatabaseHelper.Count({
			table: FRIENDS_TABLE,
			where: {
				ID_personne: userID,
				actif: 0
			}
		});
	}

	/**
	 * Count the number of friends of a specific user
	 * 
	 * @param userID Target user ID
	 */
	public static async CountForUser(userID: number) : Promise<number> {
		return await DatabaseHelper.Count({
			table: FRIENDS_TABLE,
			where: {
				ID_amis: userID,
				actif: 1
			}
		});
	}

	/**
	 * Check out whether two users are friend or not
	 * 
	 * @param userOne First user
	 * @param userTwo Second user
	 */
	public static async AreFriend(userOne: number, userTwo: number) : Promise<boolean> {
		return await DatabaseHelper.Count({
			table: FRIENDS_TABLE,
			where: {
				ID_personne: userOne,
				ID_amis: userTwo,
				actif: 1
			}
		}) > 0;
	}

	/**
	 * Check out whether friendship allows to create posts or not
	 * 
	 * @param userID User wishing to create a post
	 * @param targetID Target user ID
	 */
	public static async CanPostTexts(userID: number, targetID: number) : Promise<boolean> {
		const result = await DatabaseHelper.QueryRow({
			table: FRIENDS_TABLE,
			where: {
				ID_personne: targetID,
				ID_amis: userID,
				actif: 1
			},
			fields: ["autoriser_post_page"]
		});

		if(result == null)
			return false;
		
		return result["autoriser_post_page"] == 1;
	}

	/**
	 * Turn a database entry into a {Friend} object
	 * 
	 * @param row The row to transform
	 */
	private static DbToFriend(row: any) : Friend {
		return new Friend({
			friendID: row.ID_amis,
			accepted: row.actif == 1,
			following: row.abonnement == 1,
			lastActivityTime: row.last_activity,
			canPostTexts: row.autoriser_post_page == 1
		});
	}
}